searchData={"items":[{"type":"module","title":"Goal","doc":"Goal is a parameter validation library based on [Ecto](https://github.com/elixir-ecto/ecto).\nIt can be used with JSON APIs, HTML controllers and LiveViews.\n\nGoal builds a changeset from a validation schema and controller or LiveView parameters, and\nreturns the validated parameters or an `Ecto.Changeset`, depending on the function you use.\n\nIf your frontend and backend use different parameter cases, you can recase parameter keys with\nthe `:recase_keys` option. `PascalCase`, `camelCase`, `kebab-case` and `snake_case` are\nsupported.\n\nYou can configure your own regexes for password, email, and URL format validations. This is\nhelpful in case of backward compatibility, where Goal's defaults might not match your production\nsystem's behavior.","ref":"Goal.html"},{"type":"module","title":"Installation - Goal","doc":"Add `goal` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:goal, \"~> 0.2\"}\n  ]\nend\n```","ref":"Goal.html#module-installation"},{"type":"module","title":"Examples - Goal","doc":"Goal can be used with LiveViews and JSON and HTML controllers.","ref":"Goal.html#module-examples"},{"type":"module","title":"Example with controllers - Goal","doc":"With JSON and HTML-based APIs, Goal takes the `params` from a controller action, validates those\nagainst a validation schema using `validate/3`, and returns an atom-based map or an error\nchangeset.\n\n```elixir\ndefmodule MyApp.SomeController do\n  use MyApp, :controller\n  use Goal\n\n  def create(conn, params) do\n    with {:ok, attrs} <- validate(:create, params)) do\n      ...\n    else\n      {:error, changeset} -> {:error, changeset}\n    end\n  end\n\n  defparams :create do\n    required :uuid, :string, format: :uuid\n    required :name, :string, min: 3, max: 3\n    optional :age, :integer, min: 0, max: 120\n    optional :gender, :enum, values: [\"female\", \"male\", \"non-binary\"]\n\n    optional :data, :map do\n      required :color, :string\n      optional :money, :decimal\n      optional :height, :float\n    end\n  end\nend\n```","ref":"Goal.html#module-example-with-controllers"},{"type":"module","title":"Example with LiveViews - Goal","doc":"With LiveViews, Goal builds a changeset in `mount/3` that is assigned in the socket, and then it\ntakes the `params` from `handle_event/3`, validates those against a validation schema, and\nreturns an atom-based map or an error changeset.\n\n```elixir\ndefmodule MyApp.SomeLiveView do\n  use MyApp, :live_view\n  use Goal\n\n  def mount(params, _session, socket) do\n    changeset = changeset(:new, %{})\n    socket = assign(socket, :changeset, changeset)\n\n    {:ok, socket}\n  end\n\n  def handle_event(\"validate\", %{\"some\" => params}, socket) do\n    changeset = changeset(:new, params)\n    socket = assign(socket, :changeset, changeset)\n\n    {:noreply, socket}\n  end\n\n  def handle_event(\"save\", %{\"some\" => params}, socket) do\n    with {:ok, attrs} <- validate(:new, params)) do\n      ...\n    else\n      {:error, changeset} -> {:noreply, assign(socket, :changeset, changeset)}\n    end\n  end\n\n  defparams :new do\n    required :uuid, :string, format: :uuid\n    required :name, :string, min: 3, max: 3\n    optional :age, :integer, min: 0, max: 120\n    optional :gender, :enum, values: [\"female\", \"male\", \"non-binary\"]\n\n    optional :data, :map do\n      required :color, :string\n      optional :money, :decimal\n      optional :height, :float\n    end\n  end\nend\n```","ref":"Goal.html#module-example-with-liveviews"},{"type":"module","title":"Example with isolated schemas - Goal","doc":"Validation schemas can be defined in a separate namespace, for example `MyAppWeb.MySchema`:\n\n```elixir\ndefmodule MyAppWeb.MySchema do\n  use Goal\n\n  defparams :show do\n    required :id, :string, format: :uuid\n    optional :query, :string\n  end\nend\n\ndefmodule MyApp.SomeController do\n  use MyApp, :controller\n\n  alias MyAppWeb.MySchema\n\n  def show(conn, params) do\n    with {:ok, attrs} <- MySchema.validate(:show, params) do\n      ...\n    else\n      {:error, changeset} -> {:error, changeset}\n    end\n  end\nend\n```","ref":"Goal.html#module-example-with-isolated-schemas"},{"type":"module","title":"Features - Goal","doc":"","ref":"Goal.html#module-features"},{"type":"module","title":"Presence checks - Goal","doc":"Sometimes all you need is to check if a parameter is present:\n\n```elixir\nuse Goal\n\ndefparams :show do\n  required :id\n  optional :query\nend\n```","ref":"Goal.html#module-presence-checks"},{"type":"module","title":"Deeply nested maps - Goal","doc":"Goal efficiently builds error changesets for nested maps, and has support for lists of nested\nmaps. There is no limitation on depth.\n\n```elixir\nuse Goal\n\ndefparams :schema do\n  optional :nested_map, :map do\n    required :id, :integer\n    optional :inner_map, :map do\n      required :id, :integer\n      optional :map, :map do\n        required :id, :integer\n        optional :list, {:array, :integer}\n      end\n    end\n  end\nend\n\niex(1)> validate(:show, params)\n{:ok, %{nested_map: %{inner_map: %{map: %{id: 123, list: [1, 2, 3]}}}}}\n```","ref":"Goal.html#module-deeply-nested-maps"},{"type":"module","title":"Powerful array validations - Goal","doc":"If you need expressive validations for arrays types, look no further!\n\nArrays can be made optional/required or the number of items can be set via `min`, `max` and `is`.\nAdditionally, `rules` allows specifying any validations that are available for the inner type.\nOf course, both can be combined:\n\n```elixir\nuse Goal\n\ndefparams do\n  required :my_list, {:array, :string}, max: 2, rules: [trim: true, min: 1]\nend\n\niex(1)> Goal.validate_params(schema(), %{\"my_list\" => [\"hello \", \" world \"]})\n{:ok, %{my_list: [\"hello\", \"world\"]}}\n```","ref":"Goal.html#module-powerful-array-validations"},{"type":"module","title":"Readable error messages - Goal","doc":"Use `Goal.traverse_errors/2` to build readable errors. Phoenix by default uses\n`Ecto.Changeset.traverse_errors/2`, which works for embedded Ecto schemas but not for the plain\nnested maps used by Goal. Goal's `traverse_errors/2` is compatible with (embedded)\n`Ecto.Schema`, so you don't have to make any changes to your existing logic.\n\n```elixir\ndef translate_errors(changeset) do\n  Goal.traverse_errors(changeset, &translate_error/1)\nend\n```","ref":"Goal.html#module-readable-error-messages"},{"type":"module","title":"Recasing inbound keys - Goal","doc":"By default, Goal will look for the keys defined in `defparams`. But sometimes frontend applications\nsend parameters in a different format. For example, in `camelCase` but your backend uses\n`snake_case`. For this scenario, Goal has the `:recase_keys` option:\n\n```elixir\nconfig :goal,\n  recase_keys: [from: :camel_case]\n\niex(1)> MySchema.validate(:show, %{\"firstName\" => \"Jane\"})\n{:ok, %{first_name: \"Jane\"}}\n```","ref":"Goal.html#module-recasing-inbound-keys"},{"type":"module","title":"Recasing outbound keys - Goal","doc":"Use `recase_keys/2` to recase outbound keys. For example, in your views:\n\n```elixir\nconfig :goal,\n  recase_keys: [to: :camel_case]\n\ndefmodule MyAppWeb.UserJSON do\n  import Goal\n\n  def show(%{user: user}) do\n    recase_keys(%{data: %{first_name: user.first_name}})\n  end\n\n  def error(%{changeset: changeset}) do\n    recase_keys(%{errors: Goal.Changeset.traverse_errors(changeset, &translate_error/1)})\n  end\nend\n\niex(1)> UserJSON.show(%{user: %{first_name: \"Jane\"}})\n%{data: %{firstName: \"Jane\"}}\niex(2)> UserJSON.error(%Ecto.Changeset{errors: [first_name: {\"can't be blank\", [validation: :required]}]})\n%{errors: %{firstName: [\"can't be blank\"]}}\n```","ref":"Goal.html#module-recasing-outbound-keys"},{"type":"module","title":"Bring your own regex - Goal","doc":"Goal has sensible defaults for string format validation. If you'd like to use your own regex,\ne.g. for validating email addresses or passwords, then you can add your own regex in the\nconfiguration:\n\n```elixir\nconfig :goal,\n  uuid_regex: ~r/^[[:alpha:]]+$/,\n  email_regex: ~r/^[[:alpha:]]+$/,\n  password_regex: ~r/^[[:alpha:]]+$/,\n  url_regex: ~r/^[[:alpha:]]+$/\n```","ref":"Goal.html#module-bring-your-own-regex"},{"type":"module","title":"Available validations - Goal","doc":"The field types and available validations are:\n\n| Field type             | Validations                 | Description                                                                                          |\n| ---------------------- | --------------------------- | ---------------------------------------------------------------------------------------------------- |\n| `:uuid`                | `:equals`                   | string value                                                                                         |\n| `:string`              | `:equals`                   | string value                                                                                         |\n|                        | `:is`                       | exact string length                                                                                  |\n|                        | `:min`                      | minimum string length                                                                                |\n|                        | `:max`                      | maximum string length                                                                                |\n|                        | `:trim`                     | oolean to remove leading and trailing spaces                                                         |\n|                        | `:squish`                   | boolean to trim and collapse spaces                                                                  |\n|                        | `:format`                   | `:uuid`, `:email`, `:password`, `:url`                                                               |\n|                        | `:subset`                   | list of required strings                                                                             |\n|                        | `:included`                 | list of allowed strings                                                                              |\n|                        | `:excluded`                 | list of disallowed strings                                                                           |\n| `:integer`             | `:equals`                   | integer value                                                                                        |\n|                        | `:is`                       | integer value                                                                                        |\n|                        | `:min`                      | minimum integer value                                                                                |\n|                        | `:max`                      | maximum integer value                                                                                |\n|                        | `:greater_than`             | minimum integer value                                                                                |\n|                        | `:less_than`                | maximum integer value                                                                                |\n|                        | `:greater_than_or_equal_to` | minimum integer value                                                                                |\n|                        | `:less_than_or_equal_to`    | maximum integer value                                                                                |\n|                        | `:equal_to`                 | integer value                                                                                        |\n|                        | `:not_equal_to`             | integer value                                                                                        |\n|                        | `:subset`                   | list of required integers                                                                            |\n|                        | `:included`                 | list of allowed integers                                                                             |\n|                        | `:excluded`                 | list of disallowed integers                                                                          |\n| `:float`               |                             | all of the integer validations                                                                       |\n| `:decimal`             |                             | all of the integer validations                                                                       |\n| `:boolean`             | `:equals`                   | boolean value                                                                                        |\n| `:date`                | `:equals`                   | date value                                                                                           |\n| `:time`                | `:equals`                   | time value                                                                                           |\n| `:enum`                | `:values`                   | list of allowed values                                                                               |\n| `:map`                 | `:properties`               | use `:properties` to define the fields                                                               |\n| `{:array, :map}`       | `:properties`               | use `:properties` to define the fields                                                               |\n| `{:array, inner_type}` | `:rules`                    | `inner_type` can be any basic type. `rules` supported all validations available for `inner_type`     |\n|                        | `:min`                      | minimum array length                                                                                 |\n|                        | `:max`                      | maximum array length                                                                                 |\n|                        | `:is`                       | exact array length                                                                                   |\n| More basic types       |                             | See [Ecto.Schema](https://hexdocs.pm/ecto/Ecto.Schema.html#module-primitive-types) for the full list |\n\nAll field types, excluding `:map` and `{:array, :map}`, can use `:equals`, `:subset`,\n`:included`, `:excluded` validations.","ref":"Goal.html#module-available-validations"},{"type":"module","title":"Credits - Goal","doc":"This library is based on [Ecto](https://github.com/elixir-ecto/ecto) and I had to copy and adapt\n`Ecto.Changeset.traverse_errors/2`. Thanks for making such an awesome library! 🙇","ref":"Goal.html#module-credits"},{"type":"function","title":"Goal.build_changeset/2","doc":"Builds an `Ecto.Changeset` using the parameters and a validation schema.","ref":"Goal.html#build_changeset/2"},{"type":"function","title":"Examples - Goal.build_changeset/2","doc":"iex> build_changeset(%{email: [format: :email]}, %{\"email\" => \"jane@example.com\"})\n    %Ecto.Changeset{valid?: true, changes: %{email: \"jane@example.com\"}}\n\n    iex> build_changeset(%{email: [format: :email]}, %{\"email\" => \"invalid\"})\n    %Ecto.Changeset{valid?: false, errors: [email: {\"has invalid format\", ...}]}","ref":"Goal.html#build_changeset/2-examples"},{"type":"macro","title":"Goal.defparams/1","doc":"A macro for defining validation schemas encapsulated in a `schema` function with arity 0.\n\n```elixir\ndefmodule MySchema do\n  use Goal\n\n  defparams do\n    required :id, :string, format: :uuid\n  end\nend\n\niex(1)> schema()\n%{id: [type: :integer, required: true]}]\n```","ref":"Goal.html#defparams/1"},{"type":"macro","title":"Goal.defparams/2","doc":"A macro for defining validation schemas encapsulated in a `schema` function with arity 1.\nThe argument can be an atom or a binary.\n\n```elixir\ndefmodule MySchema do\n  use Goal\n\n  defparams :index do\n    required :id, :string, format: :uuid\n  end\nend\n\niex(1)> MySchema.schema(:index)\n%{id: [type: :integer, required: true]}]\niex(2)> MySchema.changeset(:index, %{id: 12})\n%Ecto.Changeset{valid?: true, changes: %{id: 12}}\niex(3)> MySchema.validate(:index, %{id: 12})\n{:ok, %{id: 12}}\n```","ref":"Goal.html#defparams/2"},{"type":"function","title":"Goal.recase_keys/2","doc":"Recases parameter keys.\n\nUse only when you have full control of the data. For example, to render JSON responses.","ref":"Goal.html#recase_keys/2"},{"type":"function","title":"Examples - Goal.recase_keys/2","doc":"iex> recase_keys(%{\"first_name\" => \"Jane\"}, recase_keys: [to: :camel_case])\n    %{firstName: \"Jane\"}\n\nSupported are `:camel_case`, `:pascal_case`, `:kebab_case` and `:snake_case`.","ref":"Goal.html#recase_keys/2-examples"},{"type":"function","title":"Goal.recase_keys/3","doc":"Recases parameter keys that are present in the schema.\n\nUse this instead of `recase_keys/2` for incoming parameters. For example, for user requests.","ref":"Goal.html#recase_keys/3"},{"type":"function","title":"Examples - Goal.recase_keys/3","doc":"iex> recase_keys(%{first_name: [type: :string]}, %{\"firstName\" => \"Jane\"}, recase_keys: [from: :camel_case])\n    %{first_name: \"Jane\"}\n\nSupported are `:camel_case`, `:pascal_case`, `:kebab_case` and `:snake_case`.","ref":"Goal.html#recase_keys/3-examples"},{"type":"function","title":"Goal.traverse_errors/2","doc":"Traverses changeset errors and applies the given function to error messages.","ref":"Goal.html#traverse_errors/2"},{"type":"function","title":"Examples - Goal.traverse_errors/2","doc":"iex> traverse_errors(changeset, fn {msg, opts} ->\n    ...>   Regex.replace(~r\"%{(\\w+)}\", msg, fn _, key ->\n    ...>     opts |> Keyword.get(String.to_existing_atom(key), key) |> to_string()\n    ...>   end)\n    ...> end)\n    %{title: [\"should be at least 3 characters\"]}","ref":"Goal.html#traverse_errors/2-examples"},{"type":"function","title":"Goal.validate_params/3","doc":"Validates parameters against a validation schema.","ref":"Goal.html#validate_params/3"},{"type":"function","title":"Examples - Goal.validate_params/3","doc":"iex> validate_params(%{email: [format: :email]}, %{\"email\" => \"jane@example.com\"})\n    {:ok, %{email: \"jane@example.com\"}}\n\n    iex> validate_params(%{email: [format: :email]}, %{\"email\" => \"invalid\"})\n    {:error, %Ecto.Changeset{valid?: false, errors: [email: {\"has invalid format\", ...}]}}","ref":"Goal.html#validate_params/3-examples"},{"type":"module","title":"Goal.Changeset","doc":"Goal.Changeset contains an adapted version of `Ecto.Changeset.traverse_errors/2`.\n\nYou can use `traverse_errors/2` to build all errors into a series of nested maps.","ref":"Goal.Changeset.html"},{"type":"function","title":"Goal.Changeset.traverse_errors/2","doc":"Traverses changeset errors and applies the given function to error messages.\n\nThis function is particularly useful when associations, embeds, maps\nand nested maps are cast in the changeset as it will traverse all\nassociations, embeds, maps and nested maps and place all errors in a\nseries of nested maps.\n\nA changeset is supplied along with a function to apply to each\nerror message as the changeset is traversed. The error message\nfunction receives an error tuple `{msg, opts}`, for example:\n\n    {\"should be at least %{count} characters\", [count: 3, validation: :length, min: 3]}","ref":"Goal.Changeset.html#traverse_errors/2"},{"type":"function","title":"Examples - Goal.Changeset.traverse_errors/2","doc":"iex> traverse_errors(changeset, fn {msg, opts} ->\n    ...>   Regex.replace(~r\"%{(\\w+)}\", msg, fn _, key ->\n    ...>     opts |> Keyword.get(String.to_existing_atom(key), key) |> to_string()\n    ...>   end)\n    ...> end)\n    %{title: [\"should be at least 3 characters\"]}\n\nOptionally function can accept three arguments: `changeset`, `field`\nand error tuple `{msg, opts}`. It is useful whenever you want to extract\nvalidations rules from `changeset.validations` to build detailed error\ndescription.\n\nThis function and documentation is gratefully copied and adapted\nfrom `Ecto.Changeset.traverse_errors/2` 🙇","ref":"Goal.Changeset.html#traverse_errors/2-examples"},{"type":"type","title":"Goal.Changeset.action/0","doc":"","ref":"Goal.Changeset.html#t:action/0"},{"type":"type","title":"Goal.Changeset.constraint/0","doc":"","ref":"Goal.Changeset.html#t:constraint/0"},{"type":"type","title":"Goal.Changeset.data/0","doc":"","ref":"Goal.Changeset.html#t:data/0"},{"type":"type","title":"Goal.Changeset.error/0","doc":"","ref":"Goal.Changeset.html#t:error/0"},{"type":"type","title":"Goal.Changeset.t/0","doc":"","ref":"Goal.Changeset.html#t:t/0"},{"type":"type","title":"Goal.Changeset.t/1","doc":"","ref":"Goal.Changeset.html#t:t/1"},{"type":"type","title":"Goal.Changeset.types/0","doc":"","ref":"Goal.Changeset.html#t:types/0"},{"type":"module","title":"Goal.Regex","doc":"Defines regex validations.","ref":"Goal.Regex.html"},{"type":"module","title":"Goal.String","doc":"Defines string operations.","ref":"Goal.String.html"},{"type":"function","title":"Goal.String.squish/1","doc":"Returns a string where all leading, trailing, and multiple inner Unicode whitespaces have been removed.","ref":"Goal.String.html#squish/1"},{"type":"function","title":"Examples - Goal.String.squish/1","doc":"iex> squish(\" banana  man \")\n    \"banana man\"","ref":"Goal.String.html#squish/1-examples"},{"type":"extras","title":"Goal","doc":"# Goal\n\nGoal is a parameter validation library based on [Ecto](https://github.com/elixir-ecto/ecto).\nIt can be used with JSON APIs, HTML controllers and LiveViews.\n\nGoal builds a changeset from a validation schema and controller or LiveView parameters, and\nreturns the validated parameters or `Ecto.Changeset`, depending on the function you use.\n\nIf your frontend and backend use different parameter cases, you can recase parameter keys with\nthe `:recase_keys` option. `PascalCase`, `camelCase`, `kebab-case` and `snake_case` are\nsupported.\n\nYou can configure your own regexes for password, email, and URL format validations. This is\nhelpful in case of backward compatibility, where Goal's defaults might not match your production\nsystem's behavior.","ref":"readme.html"},{"type":"extras","title":"Installation - Goal","doc":"Add `goal` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:goal, \"~> 0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - Goal","doc":"Goal can be used with LiveViews and JSON and HTML controllers.","ref":"readme.html#examples"},{"type":"extras","title":"Example with controllers - Goal","doc":"With JSON and HTML-based APIs, Goal takes the `params` from a controller action, validates those\nagainst a validation schema using `validate/3`, and returns an atom-based map or an error\nchangeset.\n\n```elixir\ndefmodule MyApp.SomeController do\n  use MyApp, :controller\n  use Goal\n\n  def create(conn, params) do\n    with {:ok, attrs} <- validate(:create, params)) do\n      ...\n    else\n      {:error, changeset} -> {:error, changeset}\n    end\n  end\n\n  defparams :create do\n    required :uuid, :string, format: :uuid\n    required :name, :string, min: 3, max: 3\n    optional :age, :integer, min: 0, max: 120\n    optional :gender, :enum, values: [\"female\", \"male\", \"non-binary\"]\n    optional :hobbies, {:array, :string}, max: 3, rules: [trim: true, min: 1]\n\n    optional :data, :map do\n      required :color, :string\n      optional :money, :decimal\n      optional :height, :float\n    end\n  end\nend\n```","ref":"readme.html#example-with-controllers"},{"type":"extras","title":"Example with LiveViews - Goal","doc":"With LiveViews, Goal builds a changeset in `mount/3` that is assigned in the socket, and then it\ntakes the `params` from `handle_event/3`, validates those against a validation schema, and\nreturns an atom-based map or an error changeset.\n\n```elixir\ndefmodule MyApp.SomeLiveView do\n  use MyApp, :live_view\n  use Goal\n\n  def mount(params, _session, socket) do\n    changeset = changeset(:new, %{})\n    socket = assign(socket, :changeset, changeset)\n\n    {:ok, socket}\n  end\n\n  def handle_event(\"validate\", %{\"some\" => params}, socket) do\n    changeset = changeset(:new, params)\n    socket = assign(socket, :changeset, changeset)\n\n    {:noreply, socket}\n  end\n\n  def handle_event(\"save\", %{\"some\" => params}, socket) do\n    with {:ok, attrs} <- validate(:new, params)) do\n      ...\n    else\n      {:error, changeset} -> {:noreply, assign(socket, :changeset, changeset)}\n    end\n  end\n\n  defparams :new do\n    required :uuid, :string, format: :uuid\n    required :name, :string, min: 3, max: 3\n    optional :age, :integer, min: 0, max: 120\n    optional :gender, :enum, values: [\"female\", \"male\", \"non-binary\"]\n    optional :hobbies, {:array, :string}, max: 3, rules: [trim: true, min: 1]\n\n    optional :data, :map do\n      required :color, :string\n      optional :money, :decimal\n      optional :height, :float\n    end\n  end\nend\n```","ref":"readme.html#example-with-liveviews"},{"type":"extras","title":"Example with isolated schemas - Goal","doc":"Validation schemas can be defined in a separate namespace, for example `MyAppWeb.MySchema`:\n\n```elixir\ndefmodule MyAppWeb.MySchema do\n  use Goal\n\n  defparams :show do\n    required :id, :string, format: :uuid\n    optional :query, :string\n  end\nend\n\ndefmodule MyApp.SomeController do\n  use MyApp, :controller\n\n  alias MyAppWeb.MySchema\n\n  def show(conn, params) do\n    with {:ok, attrs} <- MySchema.validate(:show, params) do\n      ...\n    else\n      {:error, changeset} -> {:error, changeset}\n    end\n  end\nend\n```","ref":"readme.html#example-with-isolated-schemas"},{"type":"extras","title":"Features - Goal","doc":"","ref":"readme.html#features"},{"type":"extras","title":"Presence checks - Goal","doc":"Sometimes all you need is to check if a parameter is present:\n\n```elixir\nuse Goal\n\ndefparams :show do\n  required :id\n  optional :query\nend\n```","ref":"readme.html#presence-checks"},{"type":"extras","title":"Deeply nested maps - Goal","doc":"Goal efficiently builds error changesets for nested maps, and has support for lists of nested\nmaps. There is no limitation on depth.\n\n```elixir\nuse Goal\n\ndefparams :show do\n  optional :nested_map, :map do\n    required :id, :integer\n    optional :inner_map, :map do\n      required :id, :integer\n      optional :map, :map do\n        required :id, :integer\n        optional :list, {:array, :integer}\n      end\n    end\n  end\nend\n\niex(1)> validate(:show, params)\n{:ok, %{nested_map: %{inner_map: %{map: %{id: 123, list: [1, 2, 3]}}}}}\n```","ref":"readme.html#deeply-nested-maps"},{"type":"extras","title":"Powerful array validations - Goal","doc":"If you need expressive validations for arrays types, look no further!\n\nArrays can be made optional/required or the number of items can be set via `min`, `max` and `is`.\nAdditionally, `rules` allows specifying any validations that are available for the inner type.\nOf course, both can be combined:\n\n```elixir\nuse Goal\n\ndefparams do\n  required :my_list, {:array, :string}, max: 2, rules: [trim: true, min: 1]\nend\n\niex(1)> Goal.validate_params(schema(), %{\"my_list\" => [\"hello \", \" world \"]})\n{:ok, %{my_list: [\"hello\", \"world\"]}}\n```","ref":"readme.html#powerful-array-validations"},{"type":"extras","title":"Readable error messages - Goal","doc":"Use `Goal.traverse_errors/2` to build readable errors. Phoenix by default uses\n`Ecto.Changeset.traverse_errors/2`, which works for embedded Ecto schemas but not for the plain\nnested maps used by Goal. Goal's `traverse_errors/2` is compatible with (embedded)\n`Ecto.Schema`, so you don't have to make any changes to your existing logic.\n\n```elixir\ndef translate_errors(changeset) do\n  Goal.traverse_errors(changeset, &translate_error/1)\nend\n```","ref":"readme.html#readable-error-messages"},{"type":"extras","title":"Recasing inbound keys - Goal","doc":"By default, Goal will look for the keys defined in `defparams`. But sometimes frontend applications\nsend parameters in a different format. For example, in `camelCase` but your backend uses\n`snake_case`. For this scenario, Goal has the `:recase_keys` option:\n\n```elixir\nconfig :goal,\n  recase_keys: [from: :camel_case]\n\niex(1)> MySchema.validate(:show, %{\"firstName\" => \"Jane\"})\n{:ok, %{first_name: \"Jane\"}}\n```","ref":"readme.html#recasing-inbound-keys"},{"type":"extras","title":"Recasing outbound keys - Goal","doc":"Use `recase_keys/2` to recase outbound keys. For example, in your views:\n\n```elixir\nconfig :goal,\n  recase_keys: [to: :camel_case]\n\ndefmodule MyAppWeb.UserJSON do\n  import Goal\n\n  def show(%{user: user}) do\n    recase_keys(%{data: %{first_name: user.first_name}})\n  end\n\n  def error(%{changeset: changeset}) do\n    recase_keys(%{errors: Goal.Changeset.traverse_errors(changeset, &translate_error/1)})\n  end\nend\n\niex(1)> UserJSON.show(%{user: %{first_name: \"Jane\"}})\n%{data: %{firstName: \"Jane\"}}\niex(2)> UserJSON.error(%Ecto.Changeset{errors: [first_name: {\"can't be blank\", [validation: :required]}]})\n%{errors: %{firstName: [\"can't be blank\"]}}\n```","ref":"readme.html#recasing-outbound-keys"},{"type":"extras","title":"Bring your own regex - Goal","doc":"Goal has sensible defaults for string format validation. If you'd like to use your own regex,\ne.g. for validating email addresses or passwords, then you can add your own regex in the\nconfiguration:\n\n```elixir\nconfig :goal,\n  uuid_regex: ~r/^[[:alpha:]]+$/,\n  email_regex: ~r/^[[:alpha:]]+$/,\n  password_regex: ~r/^[[:alpha:]]+$/,\n  url_regex: ~r/^[[:alpha:]]+$/\n```","ref":"readme.html#bring-your-own-regex"},{"type":"extras","title":"Available validations - Goal","doc":"The field types and available validations are:\n\n| Field type             | Validations                 | Description                                                                                          |\n| ---------------------- | --------------------------- | ---------------------------------------------------------------------------------------------------- |\n| `:uuid`                | `:equals`                   | string value                                                                                         |\n| `:string`              | `:equals`                   | string value                                                                                         |\n|                        | `:is`                       | exact string length                                                                                  |\n|                        | `:min`                      | minimum string length                                                                                |\n|                        | `:max`                      | maximum string length                                                                                |\n|                        | `:trim`                     | oolean to remove leading and trailing spaces                                                         |\n|                        | `:squish`                   | boolean to trim and collapse spaces                                                                  |\n|                        | `:format`                   | `:uuid`, `:email`, `:password`, `:url`                                                               |\n|                        | `:subset`                   | list of required strings                                                                             |\n|                        | `:included`                 | list of allowed strings                                                                              |\n|                        | `:excluded`                 | list of disallowed strings                                                                           |\n| `:integer`             | `:equals`                   | integer value                                                                                        |\n|                        | `:is`                       | integer value                                                                                        |\n|                        | `:min`                      | minimum integer value                                                                                |\n|                        | `:max`                      | maximum integer value                                                                                |\n|                        | `:greater_than`             | minimum integer value                                                                                |\n|                        | `:less_than`                | maximum integer value                                                                                |\n|                        | `:greater_than_or_equal_to` | minimum integer value                                                                                |\n|                        | `:less_than_or_equal_to`    | maximum integer value                                                                                |\n|                        | `:equal_to`                 | integer value                                                                                        |\n|                        | `:not_equal_to`             | integer value                                                                                        |\n|                        | `:subset`                   | list of required integers                                                                            |\n|                        | `:included`                 | list of allowed integers                                                                             |\n|                        | `:excluded`                 | list of disallowed integers                                                                          |\n| `:float`               |                             | all of the integer validations                                                                       |\n| `:decimal`             |                             | all of the integer validations                                                                       |\n| `:boolean`             | `:equals`                   | boolean value                                                                                        |\n| `:date`                | `:equals`                   | date value                                                                                           |\n| `:time`                | `:equals`                   | time value                                                                                           |\n| `:enum`                | `:values`                   | list of allowed values                                                                               |\n| `:map`                 | `:properties`               | use `:properties` to define the fields                                                               |\n| `{:array, :map}`       | `:properties`               | use `:properties` to define the fields                                                               |\n| `{:array, inner_type}` | `:rules`                    | `inner_type` can be any basic type. `rules` supported all validations available for `inner_type`     |\n|                        | `:min`                      | minimum array length                                                                                 |\n|                        | `:max`                      | maximum array length                                                                                 |\n|                        | `:is`                       | exact array length                                                                                   |\n| More basic types       |                             | See [Ecto.Schema](https://hexdocs.pm/ecto/Ecto.Schema.html#module-primitive-types) for the full list |\n\nAll field types, excluding `:map` and `{:array, :map}`, can use `:equals`, `:subset`,\n`:included`, `:excluded` validations.","ref":"readme.html#available-validations"},{"type":"extras","title":"Credits - Goal","doc":"This library is based on [Ecto](https://github.com/elixir-ecto/ecto) and I had to copy and adapt\n`Ecto.Changeset.traverse_errors/2`. Thanks for making such an awesome library! 🙇","ref":"readme.html#credits"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}